﻿#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file LampDataSub.py
 @brief Subscribe Lamp Data by MQTT
 @date $Date$


"""
import sys
import datetime
import json
import logging
from logging import DEBUG
import os

# Import RTM module
import RTC
import OpenRTM_aist

from mongodb import MongoDB
from postgres import PostgreSQL

sys.path.append(".")

log_dir = "./log"
if os.path.isdir(log_dir) is False:
    os.makedirs(log_dir)

if __name__ == '__main__':
    log_format = "%(asctime)s %(levelname)s %(name)s :%(message)s"
    log_file = f"{log_dir}/{datetime.datetime.now(datetime.timezone(datetime.timedelta(hours=+9), 'JST')).strftime('%Y%m%d%H%M%S')}.log"

    # stream handler setting
    stream_handler = logging.StreamHandler()
    stream_handler.setLevel(logging.INFO)
    stream_handler.setFormatter(logging.Formatter(log_format))

    # file handler setting
    file_handler = logging.FileHandler(log_file)
    file_handler.setLevel(DEBUG)
    file_handler.setFormatter(logging.Formatter(log_format))
    logging.basicConfig(level=logging.NOTSET, handlers=[stream_handler, file_handler])

logger = logging.getLogger(__name__)

DB_CONFIG_FILE_PATH = "./db_settings.json"

# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
lampdatasub_spec = ["implementation_id", "LampDataSub",
         "type_name",         "LampDataSub",
         "description",       "Subscribe Lamp Data by MQTT",
         "version",           "1.0.0",
         "vendor",            "DY",
         "category",          "IoRT",
         "activity_type",     "STATIC",
         "max_instance",      "1",
         "language",          "Python",
         "lang_type",         "SCRIPT",
         ""]
# </rtc-template>

##
# @class LampDataSub
# @brief Subscribe Lamp Data by MQTT
#
#
class LampDataSub(OpenRTM_aist.DataFlowComponentBase):

    ##
    # @brief constructor
    # @param manager Maneger Object
    #
    def __init__(self, manager):
        OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

        self._d_in = OpenRTM_aist.instantiateDataType(RTC.TimedString)
        """
        """
        self._inIn = OpenRTM_aist.InPort("in", self._d_in)





        # initialize of configuration-data.
        # <rtc-template block="init_conf_param">

        # </rtc-template>



    ##
    #
    # The initialize action (on CREATED->ALIVE transition)
    # formaer rtc_init_entry()
    #
    # @return RTC::ReturnCode_t
    #
    #
    def onInitialize(self):
        # Bind variables and configuration variable

        # Set InPort buffers
        self.addInPort("in",self._inIn)

        # Set OutPort buffers

        # Set service provider to Ports

        # Set service consumers to Ports

        # Set CORBA Service Ports

        self._mongo_client = None
        self._pg_client = None
        self._pg_cur = None

        return RTC.RTC_OK

    ###
    ##
    ## The finalize action (on ALIVE->END transition)
    ## formaer rtc_exiting_entry()
    ##
    ## @return RTC::ReturnCode_t
    #
    ##
    def onFinalize(self):

        self._db_close()

        return RTC.RTC_OK

    ###
    ##
    ## The startup action when ExecutionContext startup
    ## former rtc_starting_entry()
    ##
    ## @param ec_id target ExecutionContext Id
    ##
    ## @return RTC::ReturnCode_t
    ##
    ##
    #def onStartup(self, ec_id):
    #
    #    return RTC.RTC_OK

    ###
    ##
    ## The shutdown action when ExecutionContext stop
    ## former rtc_stopping_entry()
    ##
    ## @param ec_id target ExecutionContext Id
    ##
    ## @return RTC::ReturnCode_t
    ##
    ##
    #def onShutdown(self, ec_id):
    #
    #    return RTC.RTC_OK

    ##
    #
    # The activated action (Active state entry action)
    # former rtc_active_entry()
    #
    # @param ec_id target ExecutionContext Id
    #
    # @return RTC::ReturnCode_t
    #
    #
    def onActivated(self, ec_id):

        result, db_conf = self._is_json_file_parse(DB_CONFIG_FILE_PATH)
        if result is False:
            return RTC.RTC_ERROR

        # Create a DB instance
        # Mongo
        self._mongo_ins = MongoDB(db_conf["1st_db_host"], db_conf["1st_db_port"], db_conf["1st_db_name"], db_conf["1st_db_tbl"])
        self._mongo_client = self._mongo_ins.connection()
        # PostgreSQL
        self._pg_ins = PostgreSQL(db_conf["2nd_db_host"], db_conf["2nd_db_port"], db_conf["2nd_db_name"], db_conf["2nd_db_tbl"], db_conf["2nd_db_user"], db_conf["2nd_db_pass"])
        self._pg_client = self._pg_ins.get_connection()
        self._pg_cur = self._pg_ins.get_acquisition_cursor(self._pg_client)

        if self._mongo_client and self._pg_client:
            return RTC.RTC_OK
        else:
            return RTC.RTC_ERROR

    ##
    #
    # The deactivated action (Active state exit action)
    # former rtc_active_exit()
    #
    # @param ec_id target ExecutionContext Id
    #
    # @return RTC::ReturnCode_t
    #
    #
    def onDeactivated(self, ec_id):

        self._db_close()

        return RTC.RTC_OK

    ##
    #
    # The execution action that is invoked periodically
    # former rtc_active_do()
    #
    # @param ec_id target ExecutionContext Id
    #
    # @return RTC::ReturnCode_t
    #
    #
    def onExecute(self, ec_id):
        if self._inIn.isNew():
            data = self._inIn.read()
            print('Received', data.data)
            buf_dict = json.loads(data.data)
            for json_data in buf_dict:
                self._insert_mongo(json_data)
                self._insert_pg(json_data)

        return RTC.RTC_OK

    ###
    ##
    ## The aborting action when main logic error occurred.
    ## former rtc_aborting_entry()
    ##
    ## @param ec_id target ExecutionContext Id
    ##
    ## @return RTC::ReturnCode_t
    ##
    ##
    #def onAborting(self, ec_id):
    #
    #    return RTC.RTC_OK

    ###
    ##
    ## The error action in ERROR state
    ## former rtc_error_do()
    ##
    ## @param ec_id target ExecutionContext Id
    ##
    ## @return RTC::ReturnCode_t
    ##
    ##
    def onError(self, ec_id):

        self._db_close()
        self.reset(ec_id)

        return RTC.RTC_OK

    ###
    ##
    ## The reset action that is invoked resetting
    ## This is same but different the former rtc_init_entry()
    ##
    ## @param ec_id target ExecutionContext Id
    ##
    ## @return RTC::ReturnCode_t
    ##
    ##
    def onReset(self, ec_id):

        self._db_close()

        return RTC.RTC_OK

    ###
    ##
    ## The state update action that is invoked after onExecute() action
    ## no corresponding operation exists in OpenRTm-aist-0.2.0
    ##
    ## @param ec_id target ExecutionContext Id
    ##
    ## @return RTC::ReturnCode_t
    ##

    ##
    #def onStateUpdate(self, ec_id):
    #
    #    return RTC.RTC_OK

    ###
    ##
    ## The action that is invoked when execution context's rate is changed
    ## no corresponding operation exists in OpenRTm-aist-0.2.0
    ##
    ## @param ec_id target ExecutionContext Id
    ##
    ## @return RTC::ReturnCode_t
    ##
    ##
    #def onRateChanged(self, ec_id):
    #
    #    return RTC.RTC_OK

    def _is_json_file_parse(self, path):
        """
        Read to specified json file.

        Parameters
        ----------
        Path
            json file path

        Returns
        -------
        Parse result
            True: Success False: Failure
        """

        result = False
        json_dict = {}

        try:
            with open(path) as f:
                json_dict = json.load(f)
                if "" not in json_dict.values():
                    result = True
                else:
                    logger.error("Param is empty.Please confirm.")
        except IOError:
            logger.error("Unable to read setting file.")

        return result, json_dict

    def _insert_mongo(self, json_str) -> bool:
        """
        Insert to MongoDB.

        Parameters
        ----------
        json_str
            Insert data

        Returns
        -------
        Insert Result
        """

        rtn = False

        if self._mongo_ins.insert_document(self._mongo_ins._db_name, self._mongo_ins._collection_name, json_str):
            rtn = True

        return rtn

    def _insert_pg(self, json_str) -> bool:
        """
        Insert to PostgreSQL.

        Parameters
        ----------
        json_str
            Insert data

        Returns
        -------
        Insert Result
        """

        rtn = False

        if self._insert_lamp(json_str):
            self._pg_ins._commit()
            rtn = True
        else:
            self._pg_ins._rollback()

        return rtn

    def _insert_lamp(self, json_str) -> bool:
        """
        Insert to PostgreSQL.

        Parameters
        ----------
        json_str
            Insert data

        Returns
        -------
        Insert Result
        """

        rtn = False

        self._pg_ins._tbl_name = "ac_iort_ctrl_panel_lamp"
        INSERT_LAMP = "INSERT INTO {} (camera_id, datetime, num, color) VALUES (%s, %s, %s, %s)"
        param = (json_str["camera_id"], json_str["datetime"], json_str["lamp_num"], json_str["data"])
        if self._pg_ins.insert(self._pg_cur, INSERT_LAMP, param):
            rtn = True

        return rtn

    def _db_close(self):
        """
        DB close

        Parameters
        ----------
        None

        Returns
        -------
        None
        """

        if self._mongo_client:
            self._mongo_ins.close()
            self._mongo_client = None

        if self._pg_client:
            self._pg_ins.close()
            self._pg_cur = None
            self._pg_client = None


def LampDataSubInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=lampdatasub_spec)
    manager.registerFactory(profile,
                            LampDataSub,
                            OpenRTM_aist.Delete)

def MyModuleInit(manager):
    LampDataSubInit(manager)

    # Create a component
    comp = manager.createComponent("LampDataSub")

def main():
    mgr = OpenRTM_aist.Manager.init(sys.argv)
    mgr.setModuleInitProc(MyModuleInit)
    mgr.activateManager()
    mgr.runManager()

if __name__ == "__main__":
    main()

